<style>
  .A4 #notes { /* NOTES on top of BOM */
    position: absolute;
    {% if options.show_bom %}
    bottom: {{ (options.titleblock_rows + 1) * options.titleblock_row_height + (options.bom_rows + 2) * options.bom_row_height }}mm;
    {% else %}
    bottom: {{ (options.titleblock_rows + 1) * options.titleblock_row_height }}mm;
    {% endif %}
    right: 0;
  }

  .A3 #notes, .A2 #notes { /* NOTES on top of title block */
    position: absolute;
    {% if options.notes_on_right %}
    right: 10mm;
    bottom: {{ (options.titleblock_rows + 1) * options.titleblock_row_height }}mm;
    {% else %}
    left: 0;
    {% if options.show_bom %}
    bottom: {{ (options.bom_rows + 2) * options.bom_row_height }}mm;
    {% else %}
    bottom: 0;
    {% endif %}
    {% endif %}
  }

  #notes {
    font-size: 14px;
    font-style: italic;
  }

  .A4 #notes {
    width: {{ options.notes_width }};
  }

  .A3 #notes, .A2 #notes {
    width: {{ options.notes_width }};
    word-wrap: break-word;
    white-space: normal;
  }

  #notes ol, ul, li {
    text-align: left;
  }

</style>


<!-- NOTES TEMPLATE --!>
<div id="notes">

  <h3>Notes:</h3>
  {{ notes }}

</div>
