{% if image %}
<table border="0" cellborder="0" cellpadding="3" cellspacing="0" >
  <tr>
    {% if image.fixedsize %}
    <td balign="left">
      <table border="0" cellborder="0" cellpadding="0" cellspacing="0" >
        <tr>
          <td width="{{ image.width }}" height="{{ image.height }}" fixedsize="true"><img scale="{{ image.scale }}" src="{{ image.src }}" /></td>
        </tr>
      </table>
    </td>

    {% else %}

    <td balign="left" sides="TLR"><img scale="{{ image.scale }}" src="{{ image.src }}" /></td>

    {% endif %}
  </tr>
  {% if image.caption %}
  <tr>
    <td balign="left" sides="BLR">
      {{ image.caption }}
    </td>
  </tr>
  {% endif %}
</table>
{% else %}
<!-- No Image Defined !-->
{% endif %}
